/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletAccelerometer;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Accelerometer Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletAccelerometerActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Acceleration")
    public @ActionOutput(name = "x", type = "short") @ActionOutput(name = "y", type = "short") @ActionOutput(name = "z", type = "short") Map<String, Object> brickletAccelerometerGetAcceleration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAccelerometerWrapper dev = ((@Nullable BrickletAccelerometerWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletAccelerometer.Acceleration value = dev.getAcceleration();

        result.put("x", value.x);
        result.put("y", value.y);
        result.put("z", value.z);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerGetAcceleration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerActions) {
            return ((BrickletAccelerometerActions) actions).brickletAccelerometerGetAcceleration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerActions class.");
        }
    }

    @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type = "short") Map<String, Object> brickletAccelerometerGetTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAccelerometerWrapper dev = ((@Nullable BrickletAccelerometerWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        short value = dev.getTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerGetTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerActions) {
            return ((BrickletAccelerometerActions) actions).brickletAccelerometerGetTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerActions class.");
        }
    }

    @RuleAction(label = "LED On")
    public void brickletAccelerometerLEDOn() throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletAccelerometerWrapper dev = ((@Nullable BrickletAccelerometerWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.ledOn();
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletAccelerometerLED")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletAccelerometerLED"), RefreshType.REFRESH);
        }
    }

    public static void brickletAccelerometerLEDOn(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerActions) {
            ((BrickletAccelerometerActions) actions).brickletAccelerometerLEDOn();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerActions class.");
        }
    }

    @RuleAction(label = "LED Off")
    public void brickletAccelerometerLEDOff() throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletAccelerometerWrapper dev = ((@Nullable BrickletAccelerometerWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.ledOff();
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletAccelerometerLED")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletAccelerometerLED"), RefreshType.REFRESH);
        }
    }

    public static void brickletAccelerometerLEDOff(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerActions) {
            ((BrickletAccelerometerActions) actions).brickletAccelerometerLEDOff();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerActions class.");
        }
    }

    @RuleAction(label = "Is LED On")
    public @ActionOutput(name = "on", type = "boolean") Map<String, Object> brickletAccelerometerIsLEDOn()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAccelerometerWrapper dev = ((@Nullable BrickletAccelerometerWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean value = dev.isLEDOn();

        result.put("on", value);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerIsLEDOn(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerActions) {
            return ((BrickletAccelerometerActions) actions).brickletAccelerometerIsLEDOn();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerActions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "dataRate", type = "short") @ActionOutput(name = "fullScale", type = "short") @ActionOutput(name = "filterBandwidth", type = "short") Map<String, Object> brickletAccelerometerGetConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAccelerometerWrapper dev = ((@Nullable BrickletAccelerometerWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletAccelerometer.Configuration value = dev.getConfiguration();

        result.put("dataRate", value.dataRate);
        result.put("fullScale", value.fullScale);
        result.put("filterBandwidth", value.filterBandwidth);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerGetConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerActions) {
            return ((BrickletAccelerometerActions) actions).brickletAccelerometerGetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletAccelerometerGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAccelerometerWrapper dev = ((@Nullable BrickletAccelerometerWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletAccelerometer.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerActions) {
            return ((BrickletAccelerometerActions) actions).brickletAccelerometerGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerActions class.");
        }
    }
}
