/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletAmbientLightV3;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Ambient Light Bricklet 3.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletAmbientLightV3Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Illuminance")
    public @ActionOutput(name = "illuminance", type = "long") Map<String, Object> brickletAmbientLightV3GetIlluminance()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAmbientLightV3Wrapper dev = ((@Nullable BrickletAmbientLightV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.getIlluminance();

        result.put("illuminance", value);
        return result;
    }

    public static Map<String, Object> brickletAmbientLightV3GetIlluminance(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAmbientLightV3Actions) {
            return ((BrickletAmbientLightV3Actions) actions).brickletAmbientLightV3GetIlluminance();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAmbientLightV3Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "illuminanceRange", type = "int") @ActionOutput(name = "integrationTime", type = "int") Map<String, Object> brickletAmbientLightV3GetConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAmbientLightV3Wrapper dev = ((@Nullable BrickletAmbientLightV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletAmbientLightV3.Configuration value = dev.getConfiguration();

        result.put("illuminanceRange", value.illuminanceRange);
        result.put("integrationTime", value.integrationTime);
        return result;
    }

    public static Map<String, Object> brickletAmbientLightV3GetConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAmbientLightV3Actions) {
            return ((BrickletAmbientLightV3Actions) actions).brickletAmbientLightV3GetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAmbientLightV3Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletAmbientLightV3ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAmbientLightV3Wrapper dev = ((@Nullable BrickletAmbientLightV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletAmbientLightV3ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAmbientLightV3Actions) {
            return ((BrickletAmbientLightV3Actions) actions).brickletAmbientLightV3ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAmbientLightV3Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletAmbientLightV3GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAmbientLightV3Wrapper dev = ((@Nullable BrickletAmbientLightV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletAmbientLightV3GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAmbientLightV3Actions) {
            return ((BrickletAmbientLightV3Actions) actions).brickletAmbientLightV3GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAmbientLightV3Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletAmbientLightV3GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAmbientLightV3Wrapper dev = ((@Nullable BrickletAmbientLightV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletAmbientLightV3GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAmbientLightV3Actions) {
            return ((BrickletAmbientLightV3Actions) actions).brickletAmbientLightV3GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAmbientLightV3Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletAmbientLightV3GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAmbientLightV3Wrapper dev = ((@Nullable BrickletAmbientLightV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletAmbientLightV3.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletAmbientLightV3GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAmbientLightV3Actions) {
            return ((BrickletAmbientLightV3Actions) actions).brickletAmbientLightV3GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAmbientLightV3Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "int[]") @ActionOutput(name = "firmwareVersion", type = "int[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletAmbientLightV3GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAmbientLightV3Wrapper dev = ((@Nullable BrickletAmbientLightV3Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletAmbientLightV3.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletAmbientLightV3GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAmbientLightV3Actions) {
            return ((BrickletAmbientLightV3Actions) actions).brickletAmbientLightV3GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAmbientLightV3Actions class.");
        }
    }
}
