/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletAnalogOut;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Analog Out Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletAnalogOutActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Set Voltage")
    public void brickletAnalogOutSetVoltage(@ActionInput(name = "voltage") int voltage) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletAnalogOutWrapper dev = ((@Nullable BrickletAnalogOutWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setVoltage(voltage);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletAnalogOutVoltage")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletAnalogOutVoltage"), RefreshType.REFRESH);
        }
    }

    public static void brickletAnalogOutSetVoltage(@Nullable ThingActions actions, int voltage)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutActions) {
            ((BrickletAnalogOutActions) actions).brickletAnalogOutSetVoltage(voltage);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type = "int") Map<String, Object> brickletAnalogOutGetVoltage()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAnalogOutWrapper dev = ((@Nullable BrickletAnalogOutWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getVoltage();

        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutGetVoltage(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutActions) {
            return ((BrickletAnalogOutActions) actions).brickletAnalogOutGetVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Set Mode")
    public void brickletAnalogOutSetMode(@ActionInput(name = "mode") short mode) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletAnalogOutWrapper dev = ((@Nullable BrickletAnalogOutWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setMode(mode);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletAnalogOutMode")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletAnalogOutMode"), RefreshType.REFRESH);
        }
    }

    public static void brickletAnalogOutSetMode(@Nullable ThingActions actions, short mode)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutActions) {
            ((BrickletAnalogOutActions) actions).brickletAnalogOutSetMode(mode);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Mode")
    public @ActionOutput(name = "mode", type = "short") Map<String, Object> brickletAnalogOutGetMode()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAnalogOutWrapper dev = ((@Nullable BrickletAnalogOutWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        short value = dev.getMode();

        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutGetMode(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutActions) {
            return ((BrickletAnalogOutActions) actions).brickletAnalogOutGetMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletAnalogOutGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletAnalogOutWrapper dev = ((@Nullable BrickletAnalogOutWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletAnalogOut.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutActions) {
            return ((BrickletAnalogOutActions) actions).brickletAnalogOutGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutActions class.");
        }
    }
}
