/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletDistanceIRV2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Distance IR Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletDistanceIRV2Wrapper extends BrickletDistanceIRV2 implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletDistanceIRV2.DEVICE_DISPLAY_NAME,
            "brickletdistanceirv2", BrickletDistanceIRV2.DEVICE_IDENTIFIER, BrickletDistanceIRV2Wrapper.class,
            BrickletDistanceIRV2Actions.class, "2.0.0", true);

    public BrickletDistanceIRV2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletDistanceIRV2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletDistanceIRV2Config cfg = (BrickletDistanceIRV2Config) config.as(BrickletDistanceIRV2Config.class);
        this.addDistanceListener(this.reg((distance) -> {
            if (true) {
                updateStateFn.accept("BrickletDistanceIRV2Distance",
                        transformBrickletDistanceIRV2DistanceCallback0(distance, cfg));
            }
        }, this::removeDistanceListener));
        this.addAnalogValueListener(this.reg((analogValue) -> {
            if (true) {
                updateStateFn.accept("BrickletDistanceIRV2AnalogValue",
                        transformBrickletDistanceIRV2AnalogValueCallback0(analogValue, cfg));
            }
        }, this::removeAnalogValueListener));
        this.setMovingAverageConfiguration(cfg.movingAverageLength);
        this.setDistanceLEDConfig(cfg.distanceLEDConfig);
        this.setStatusLEDConfig(cfg.statusLEDConfig);
        {
            BrickletDistanceIRV2DistanceConfig channelCfg = getChannelConfigFn.apply("BrickletDistanceIRV2Distance")
                    .as(BrickletDistanceIRV2DistanceConfig.class);
            this.setDistanceCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
        }
        {
            BrickletDistanceIRV2AnalogValueConfig channelCfg = getChannelConfigFn
                    .apply("BrickletDistanceIRV2AnalogValue").as(BrickletDistanceIRV2AnalogValueConfig.class);
            this.setAnalogValueCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setDistanceCallbackConfiguration(0, true, 'x', 0, 0);
        this.setAnalogValueCallbackConfiguration(0, true, 'x', 0, 0);

    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletDistanceIRV2Distance");
        result.add("BrickletDistanceIRV2AnalogValue");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletDistanceIRV2Distance":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletDistanceIRV2Distance"), "Distance",
                                "Number:Length")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletDistanceIRV2Distance"))
                        .withDescription("Measured distance").withStateDescription(StateDescriptionFragmentBuilder
                                .create().withPattern("%.3f %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletDistanceIRV2AnalogValue":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletDistanceIRV2AnalogValue"), "Analog Value",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletDistanceIRV2AnalogValue"))
                        .withDescription(
                                "The analog value as read by a analog-to-digital converter.<br/><br/>This is unfiltered raw data. We made sure that the integration time of the ADC is shorter then the measurement interval of the sensor (10ms vs 16.5ms). So there is no information lost.<br/><br/>If you want to do your own calibration or create your own lookup table you can use this value.")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(2097151)).withStep(BigDecimal.valueOf(1))
                                        .withPattern("%d").withReadOnly(true).build().toStateDescription())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Distance IR Bricklet 2.0").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(
                        URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Measures distance up to 150cm with infrared light")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletDistanceIRV2Distance",
                                new ChannelTypeUID("tinkerforge", "BrickletDistanceIRV2Distance")).withLabel("Distance")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletDistanceIRV2AnalogValue",
                                new ChannelTypeUID("tinkerforge", "BrickletDistanceIRV2AnalogValue"))
                                        .withLabel("Analog Value").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletdistanceirv2":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("movingAverageLength", Type.INTEGER).withDefault("25")
                                .withDescription(
                                        "Sets the length of a moving averaging for the distance.<br/><br/>Setting the length to 1 will turn the averaging off. With less averaging, there is more noise on the data.<br/><br/>The range for the averaging is 1-1000.<br/><br/>New data is gathered every ~10ms. With a moving average of length 1000 the resulting averaging window has a length of approximately 10s. If you want to do long term measurements the longest moving average will give the cleanest results.<br/><br/>The default value is 25.")
                                .withLabel("Moving Average Length").withMinimum(BigDecimal.valueOf(1))
                                .withMaximum(BigDecimal.valueOf(1000)).build(),
                        ConfigDescriptionParameterBuilder.create("distanceLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Configures the distance LED to be either turned off, turned on, blink in heartbeat mode or show the distance (brighter = object is nearer).")
                                .withLabel("Distance LED Config").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Distance")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletDistanceIRV2Distance":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletDistanceIRV2AnalogValue":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletDistanceIRV2Config cfg = (BrickletDistanceIRV2Config) config.as(BrickletDistanceIRV2Config.class);
        switch (channel) {
            case "BrickletDistanceIRV2Distance": {
                int value = this.getDistance();
                if (true)
                    updateStateFn.accept(channel, transformBrickletDistanceIRV2DistanceGetter0(value, cfg));
            }
                return;
            case "BrickletDistanceIRV2AnalogValue": {
                long value = this.getAnalogValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletDistanceIRV2AnalogValueGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletDistanceIRV2DistanceCallback0(int distance,
            BrickletDistanceIRV2Config cfg) {
        return new QuantityType<>(distance / 1000.0, SIUnits.METRE);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletDistanceIRV2AnalogValueCallback0(long analogValue,
            BrickletDistanceIRV2Config cfg) {
        return new DecimalType(analogValue);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletDistanceIRV2DistanceGetter0(int value,
            BrickletDistanceIRV2Config cfg) {
        return new QuantityType<>(value / 1000.0, SIUnits.METRE);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletDistanceIRV2AnalogValueGetter0(long value,
            BrickletDistanceIRV2Config cfg) {
        return new DecimalType(value);
    }

}
