/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletDualButton;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Dual Button Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletDualButtonActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get LED State")
    public @ActionOutput(name = "ledL", type = "short") @ActionOutput(name = "ledR", type = "short") Map<String, Object> brickletDualButtonGetLEDState()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDualButtonWrapper dev = ((@Nullable BrickletDualButtonWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletDualButton.LEDState value = dev.getLEDState();

        result.put("ledL", value.ledL);
        result.put("ledR", value.ledR);
        return result;
    }

    public static Map<String, Object> brickletDualButtonGetLEDState(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletDualButtonActions) {
            return ((BrickletDualButtonActions) actions).brickletDualButtonGetLEDState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualButtonActions class.");
        }
    }

    @RuleAction(label = "Get Button State")
    public @ActionOutput(name = "buttonL", type = "short") @ActionOutput(name = "buttonR", type = "short") Map<String, Object> brickletDualButtonGetButtonState()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDualButtonWrapper dev = ((@Nullable BrickletDualButtonWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletDualButton.ButtonState value = dev.getButtonState();

        result.put("buttonL", value.buttonL);
        result.put("buttonR", value.buttonR);
        return result;
    }

    public static Map<String, Object> brickletDualButtonGetButtonState(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletDualButtonActions) {
            return ((BrickletDualButtonActions) actions).brickletDualButtonGetButtonState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualButtonActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletDualButtonGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDualButtonWrapper dev = ((@Nullable BrickletDualButtonWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletDualButton.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletDualButtonGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletDualButtonActions) {
            return ((BrickletDualButtonActions) actions).brickletDualButtonGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualButtonActions class.");
        }
    }
}
