/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletHallEffect;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Hall Effect Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletHallEffectActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Value")
    public @ActionOutput(name = "value", type = "boolean") Map<String, Object> brickletHallEffectGetValue()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletHallEffectWrapper dev = ((@Nullable BrickletHallEffectWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean value = dev.getValue();

        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletHallEffectGetValue(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectActions) {
            return ((BrickletHallEffectActions) actions).brickletHallEffectGetValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectActions class.");
        }
    }

    @RuleAction(label = "Get Edge Count")
    public @ActionOutput(name = "count", type = "long") Map<String, Object> brickletHallEffectGetEdgeCount(
            @ActionInput(name = "resetCounter") boolean resetCounter) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletHallEffectWrapper dev = ((@Nullable BrickletHallEffectWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.getEdgeCount(resetCounter);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletHallEffectEdgeCount")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletHallEffectEdgeCount"), RefreshType.REFRESH);
        }
        result.put("count", value);
        return result;
    }

    public static Map<String, Object> brickletHallEffectGetEdgeCount(@Nullable ThingActions actions,
            boolean resetCounter) throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectActions) {
            return ((BrickletHallEffectActions) actions).brickletHallEffectGetEdgeCount(resetCounter);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectActions class.");
        }
    }

    @RuleAction(label = "Get Edge Count Config")
    public @ActionOutput(name = "edgeType", type = "short") @ActionOutput(name = "debounce", type = "short") Map<String, Object> brickletHallEffectGetEdgeCountConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletHallEffectWrapper dev = ((@Nullable BrickletHallEffectWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletHallEffect.EdgeCountConfig value = dev.getEdgeCountConfig();

        result.put("edgeType", value.edgeType);
        result.put("debounce", value.debounce);
        return result;
    }

    public static Map<String, Object> brickletHallEffectGetEdgeCountConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectActions) {
            return ((BrickletHallEffectActions) actions).brickletHallEffectGetEdgeCountConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletHallEffectGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletHallEffectWrapper dev = ((@Nullable BrickletHallEffectWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletHallEffect.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletHallEffectGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectActions) {
            return ((BrickletHallEffectActions) actions).brickletHallEffectGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectActions class.");
        }
    }
}
