/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.OnOffType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletHumidityV2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Humidity Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletHumidityV2Wrapper extends BrickletHumidityV2 implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletHumidityV2.DEVICE_DISPLAY_NAME,
            "bricklethumidityv2", BrickletHumidityV2.DEVICE_IDENTIFIER, BrickletHumidityV2Wrapper.class,
            BrickletHumidityV2Actions.class, "2.0.3", true);

    public BrickletHumidityV2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletHumidityV2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletHumidityV2Config cfg = (BrickletHumidityV2Config) config.as(BrickletHumidityV2Config.class);
        this.addHumidityListener(this.reg((humidity) -> {
            if (true) {
                updateStateFn.accept("BrickletHumidityV2Humidity",
                        transformBrickletHumidityV2HumidityCallback0(humidity, cfg));
            }
        }, this::removeHumidityListener));
        this.addTemperatureListener(this.reg((temperature) -> {
            if (true) {
                updateStateFn.accept("BrickletHumidityV2Temperature",
                        transformBrickletHumidityV2TemperatureCallback0(temperature, cfg));
            }
        }, this::removeTemperatureListener));
        this.setSamplesPerSecond(cfg.sampleRate);
        this.setMovingAverageConfiguration(cfg.humidityMovingAverageLength, cfg.temperatureMovingAverageLength);
        this.setStatusLEDConfig(cfg.statusLEDConfig);
        {
            BrickletHumidityV2HumidityConfig channelCfg = getChannelConfigFn.apply("BrickletHumidityV2Humidity")
                    .as(BrickletHumidityV2HumidityConfig.class);
            this.setHumidityCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
        }
        {
            BrickletHumidityV2TemperatureConfig channelCfg = getChannelConfigFn.apply("BrickletHumidityV2Temperature")
                    .as(BrickletHumidityV2TemperatureConfig.class);
            this.setTemperatureCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setHumidityCallbackConfiguration(0, true, 'x', 0, 0);
        this.setTemperatureCallbackConfiguration(0, true, 'x', 0, 0);

    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletHumidityV2Humidity");
        result.add("BrickletHumidityV2Temperature");
        result.add("BrickletHumidityV2Heater");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletHumidityV2Humidity":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletHumidityV2Humidity"), "Humidity",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletHumidityV2Humidity"))
                        .withDescription("Measured relative humidity")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(100)).withStep(BigDecimal.valueOf(0.01))
                                        .withPattern("%.2f %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletHumidityV2Temperature":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletHumidityV2Temperature"), "Temperature",
                                "Number:Temperature")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletHumidityV2Temperature"))
                        .withDescription("Measured temperature")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(-40))
                                        .withMaximum(BigDecimal.valueOf(165)).withStep(BigDecimal.valueOf(0.01))
                                        .withPattern("%.2f %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletHumidityV2Heater":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletHumidityV2Heater"), "Heater", "Switch")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletHumidityV2Heater"))
                        .withDescription(
                                "Enables/disables the heater. The heater can be used to dry the sensor in extremely wet conditions.")
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Humidity Bricklet 2.0").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Measures relative humidity")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletHumidityV2Humidity",
                                new ChannelTypeUID("tinkerforge", "BrickletHumidityV2Humidity")).withLabel("Humidity")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletHumidityV2Temperature",
                                new ChannelTypeUID("tinkerforge", "BrickletHumidityV2Temperature"))
                                        .withLabel("Temperature").build(),
                        new ChannelDefinitionBuilder("BrickletHumidityV2Heater",
                                new ChannelTypeUID("tinkerforge", "BrickletHumidityV2Heater")).withLabel("Heater")
                                        .build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:bricklethumidityv2":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(
                                ConfigDescriptionParameterBuilder.create("humidityMovingAverageLength", Type.INTEGER)
                                        .withDefault("5")
                                        .withDescription("The length of a moving averaging for the humidity.")
                                        .withGroupName("average").withLabel("Humidity Moving Average Length")
                                        .withMinimum(BigDecimal.valueOf(1)).withMaximum(
                                                BigDecimal.valueOf(1000))
                                        .build(),
                                ConfigDescriptionParameterBuilder.create("temperatureMovingAverageLength", Type.INTEGER)
                                        .withDefault("5")
                                        .withDescription("The length of a moving averaging for the temperature.")
                                        .withGroupName("average").withLabel("Temperature Moving Average Length")
                                        .withMinimum(BigDecimal.valueOf(1)).withMaximum(BigDecimal.valueOf(1000))
                                        .build(),
                                ConfigDescriptionParameterBuilder.create("sampleRate", Type.INTEGER).withDefault("3")
                                        .withDescription(
                                                "The samples per second that are gathered by the humidity/temperature sensor HDC1080.<br/><br/>We added this function since we found out that a high measurement frequency can lead to self-heating of the sensor. Which can distort the temperature measurement.<br/><br/>If you don't need a lot of measurements, you can use the lowest available measurement frequency of 0.1 samples per second for the least amount of self-heating.")
                                        .withLabel("Sample Rate").withLimitToOptions(true)
                                        .withOptions(Arrays.asList(new ParameterOption("0", "20 SPS"),
                                                new ParameterOption("1", "10 SPS"), new ParameterOption("2", "5 SPS"),
                                                new ParameterOption("3", "1 SPS"), new ParameterOption("4", "0.2 SPS"),
                                                new ParameterOption("5", "0.1 SPS")))
                                        .build(),
                                ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER)
                                        .withDefault("3")
                                        .withDescription(
                                                "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                        .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                        .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                                new ParameterOption("1", "On"),
                                                new ParameterOption("2", "Show Heartbeat"),
                                                new ParameterOption("3", "Show Status")))
                                        .build()))
                        .withParameterGroups(Arrays.asList(
                                new ConfigDescriptionParameterGroup("update_intervals", "null", true,
                                        "Update Intervals", "Update Intervals"),
                                new ConfigDescriptionParameterGroup("average", "null", true, "Averaging",
                                        "Sets the length of a moving averaging for the humidity and temperature.<br/><br/>Setting the length to 1 will turn the averaging off. With less averaging, there is more noise on the data.<br/><br/>The range for the averaging is 1-1000.<br/><br/>New data is gathered every 50ms*. With a moving average of length 1000 the resulting averaging window has a length of 50s. If you want to do long term measurements the longest moving average will give the cleanest results.<br/><br/>The default value is 5.<br/><br/>* In firmware version 2.0.3 we added the setSamplesPerSecond() function. It configures the measurement frequency. Since high frequencies can result in self-heating of th IC, changed the default value from 20 samples per second to 1. With 1 sample per second a moving average length of 1000 would result in an averaging window of 1000 seconds!")))
                        .build();
            case "channel-type:tinkerforge:BrickletHumidityV2Humidity":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletHumidityV2Temperature":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletHumidityV2Heater":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletHumidityV2Config cfg = (BrickletHumidityV2Config) config.as(BrickletHumidityV2Config.class);
        switch (channel) {
            case "BrickletHumidityV2Humidity": {
                int value = this.getHumidity();
                if (true)
                    updateStateFn.accept(channel, transformBrickletHumidityV2HumidityGetter0(value, cfg));
            }
                return;
            case "BrickletHumidityV2Temperature": {
                int value = this.getTemperature();
                if (true)
                    updateStateFn.accept(channel, transformBrickletHumidityV2TemperatureGetter0(value, cfg));
            }
                return;
            case "BrickletHumidityV2Heater": {
                int value = this.getHeaterConfiguration();
                if (true)
                    updateStateFn.accept(channel, transformBrickletHumidityV2HeaterGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            case "BrickletHumidityV2Heater":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setHeaterConfiguration(cmd == OnOffType.ON ? BrickletHumidityV2.HEATER_CONFIG_ENABLED
                            : BrickletHumidityV2.HEATER_CONFIG_DISABLED);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletHumidityV2HumidityCallback0(int humidity,
            BrickletHumidityV2Config cfg) {
        return new QuantityType<>(humidity / 100.0, SmartHomeUnits.PERCENT);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletHumidityV2TemperatureCallback0(int temperature,
            BrickletHumidityV2Config cfg) {
        return new QuantityType<>(temperature / 100.0, SIUnits.CELSIUS);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletHumidityV2HumidityGetter0(int value,
            BrickletHumidityV2Config cfg) {
        return new QuantityType<>(value / 100.0, SmartHomeUnits.PERCENT);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletHumidityV2TemperatureGetter0(int value,
            BrickletHumidityV2Config cfg) {
        return new QuantityType<>(value / 100.0, SIUnits.CELSIUS);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletHumidityV2HeaterGetter0(int value,
            BrickletHumidityV2Config cfg) {
        return value == BrickletHumidityV2.HEATER_CONFIG_ENABLED ? OnOffType.ON : OnOffType.OFF;
    }

}
