/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletIndustrialAnalogOut;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Industrial Analog Out Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialAnalogOutActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Enable")
    public void brickletIndustrialAnalogOutEnable() throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletIndustrialAnalogOutWrapper dev = ((@Nullable BrickletIndustrialAnalogOutWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.enable();
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialAnalogOutEnabled")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialAnalogOutEnabled"), RefreshType.REFRESH);
        }
    }

    public static void brickletIndustrialAnalogOutEnable(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutActions) {
            ((BrickletIndustrialAnalogOutActions) actions).brickletIndustrialAnalogOutEnable();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Disable")
    public void brickletIndustrialAnalogOutDisable() throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletIndustrialAnalogOutWrapper dev = ((@Nullable BrickletIndustrialAnalogOutWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.disable();
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialAnalogOutEnabled")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialAnalogOutEnabled"), RefreshType.REFRESH);
        }
    }

    public static void brickletIndustrialAnalogOutDisable(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutActions) {
            ((BrickletIndustrialAnalogOutActions) actions).brickletIndustrialAnalogOutDisable();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Is Enabled")
    public @ActionOutput(name = "enabled", type = "boolean") Map<String, Object> brickletIndustrialAnalogOutIsEnabled()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialAnalogOutWrapper dev = ((@Nullable BrickletIndustrialAnalogOutWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean value = dev.isEnabled();

        result.put("enabled", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutIsEnabled(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutActions) {
            return ((BrickletIndustrialAnalogOutActions) actions).brickletIndustrialAnalogOutIsEnabled();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type = "int") Map<String, Object> brickletIndustrialAnalogOutGetVoltage()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialAnalogOutWrapper dev = ((@Nullable BrickletIndustrialAnalogOutWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getVoltage();

        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutGetVoltage(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutActions) {
            return ((BrickletIndustrialAnalogOutActions) actions).brickletIndustrialAnalogOutGetVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Current")
    public @ActionOutput(name = "current", type = "int") Map<String, Object> brickletIndustrialAnalogOutGetCurrent()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialAnalogOutWrapper dev = ((@Nullable BrickletIndustrialAnalogOutWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getCurrent();

        result.put("current", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutGetCurrent(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutActions) {
            return ((BrickletIndustrialAnalogOutActions) actions).brickletIndustrialAnalogOutGetCurrent();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "voltageRange", type = "short") @ActionOutput(name = "currentRange", type = "short") Map<String, Object> brickletIndustrialAnalogOutGetConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialAnalogOutWrapper dev = ((@Nullable BrickletIndustrialAnalogOutWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletIndustrialAnalogOut.Configuration value = dev.getConfiguration();

        result.put("voltageRange", value.voltageRange);
        result.put("currentRange", value.currentRange);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutGetConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutActions) {
            return ((BrickletIndustrialAnalogOutActions) actions).brickletIndustrialAnalogOutGetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletIndustrialAnalogOutGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialAnalogOutWrapper dev = ((@Nullable BrickletIndustrialAnalogOutWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletIndustrialAnalogOut.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutActions) {
            return ((BrickletIndustrialAnalogOutActions) actions).brickletIndustrialAnalogOutGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutActions class.");
        }
    }
}
