/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.thing.CommonTriggerEvents;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletJoystick;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Joystick Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletJoystickWrapper extends BrickletJoystick implements StandardFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletJoystick.DEVICE_DISPLAY_NAME,
            "brickletjoystick", BrickletJoystick.DEVICE_IDENTIFIER, BrickletJoystickWrapper.class,
            BrickletJoystickActions.class, "2.0.0", false);

    public BrickletJoystickWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletJoystickWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletJoystickConfig cfg = (BrickletJoystickConfig) config.as(BrickletJoystickConfig.class);
        this.addPositionListener(this.reg((x, y) -> {
            if (true) {
                updateStateFn.accept("BrickletJoystickPositionX",
                        transformBrickletJoystickPositionXCallback0(x, y, cfg));
            }
        }, this::removePositionListener));
        this.addPositionListener(this.reg((x, y) -> {
            if (true) {
                updateStateFn.accept("BrickletJoystickPositionY",
                        transformBrickletJoystickPositionYCallback0(x, y, cfg));
            }
        }, this::removePositionListener));
        this.addPressedListener(this.reg(() -> {
            if (true) {
                triggerChannelFn.accept("BrickletJoystickPressed", transformBrickletJoystickPressedCallback0(cfg));
            }
        }, this::removePressedListener));
        this.addReleasedListener(this.reg(() -> {
            if (true) {
                triggerChannelFn.accept("BrickletJoystickPressed", transformBrickletJoystickPressedCallback1(cfg));
            }
        }, this::removeReleasedListener));
        this.addAnalogValueListener(this.reg((x, y) -> {
            if (true) {
                updateStateFn.accept("BrickletJoystickAnalogValueX",
                        transformBrickletJoystickAnalogValueXCallback0(x, y, cfg));
            }
        }, this::removeAnalogValueListener));
        this.addAnalogValueListener(this.reg((x, y) -> {
            if (true) {
                updateStateFn.accept("BrickletJoystickAnalogValueY",
                        transformBrickletJoystickAnalogValueYCallback0(x, y, cfg));
            }
        }, this::removeAnalogValueListener));
        this.setPositionCallbackPeriod(cfg.positionUpdateInterval);
        this.setAnalogValueCallbackPeriod(cfg.analogValueUpdateInterval);
        this.setAnalogValueCallbackThreshold('x', 0, 0, 0, 0);

    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setPositionCallbackPeriod(0);
        this.setAnalogValueCallbackPeriod(0);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletJoystickPositionX");
        result.add("BrickletJoystickPositionY");
        result.add("BrickletJoystickPressed");
        result.add("BrickletJoystickAnalogValueX");
        result.add("BrickletJoystickAnalogValueY");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletJoystickPosition":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletJoystickPosition"), "Position",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletJoystickPosition"))
                        .withDescription(
                                "The position of the joystick. The value ranges between -100 and 100 for both axis. The middle position of the joystick is x=0, y=0. The returned values are averaged and calibrated.")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(-100))
                                        .withMaximum(BigDecimal.valueOf(100)).withStep(BigDecimal.valueOf(1))
                                        .withPattern("%d").withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletJoystickAnalogValueX":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletJoystickAnalogValueX"), "Analog Value X",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletJoystickAnalogValueX"))
                        .withDescription(
                                "<br/>Returns the X value as read by a 12-bit analog-to-digital converter.<br/><br/>.. note::<br/> The values returned by :func:`Get Position` are averaged over several samples<br/> to yield less noise, while :func:`Get Analog Value` gives back raw<br/> unfiltered analog values. The only reason to use :func:`Get Analog Value` is,<br/> if you need the full resolution of the analog-to-digital converter.<br/>")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(4095)).withStep(BigDecimal.valueOf(1))
                                        .withPattern("%d").withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletJoystickAnalogValueY":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletJoystickAnalogValueY"), "Analog Value Y",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletJoystickAnalogValueY"))
                        .withDescription(
                                "<br/>Returns the Y value as read by a 12-bit analog-to-digital converter.<br/><br/>.. note::<br/> The values returned by :func:`Get Position` are averaged over several samples<br/> to yield less noise, while :func:`Get Analog Value` gives back raw<br/> unfiltered analog values. The only reason to use :func:`Get Analog Value` is,<br/> if you need the full resolution of the analog-to-digital converter.<br/>")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(4095)).withStep(BigDecimal.valueOf(1))
                                        .withPattern("%d").withReadOnly(true).build().toStateDescription())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Joystick Bricklet").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("2-axis joystick with push-button")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletJoystickPositionX",
                                new ChannelTypeUID("tinkerforge", "BrickletJoystickPosition")).withLabel("Position X")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletJoystickPositionY",
                                new ChannelTypeUID("tinkerforge", "BrickletJoystickPosition")).withLabel("Position Y")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletJoystickPressed",
                                new ChannelTypeUID("system", "rawbutton")).withLabel("Pressed")
                                        .withDescription("Triggers if the button is pressed or released").build(),
                        new ChannelDefinitionBuilder("BrickletJoystickAnalogValueX",
                                new ChannelTypeUID("tinkerforge", "BrickletJoystickAnalogValueX"))
                                        .withLabel("Analog Value X").build(),
                        new ChannelDefinitionBuilder("BrickletJoystickAnalogValueY",
                                new ChannelTypeUID("tinkerforge", "BrickletJoystickAnalogValueY"))
                                        .withLabel("Analog Value Y").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletjoystick":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("positionUpdateInterval", Type.INTEGER)
                                .withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval for X and Y position in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Position Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("analogValueUpdateInterval", Type.INTEGER)
                                .withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval for X and Y analog value in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Analog Value Update Interval")
                                .withUnit("ms").withMinimum(BigDecimal.valueOf(0))
                                .withMaximum(BigDecimal.valueOf(4294967295L)).build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletJoystickPosition":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletJoystickAnalogValueX":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletJoystickAnalogValueY":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletJoystickConfig cfg = (BrickletJoystickConfig) config.as(BrickletJoystickConfig.class);
        switch (channel) {
            case "BrickletJoystickPositionX": {
                BrickletJoystick.Position value = this.getPosition();
                if (true)
                    updateStateFn.accept(channel, transformBrickletJoystickPositionXGetter0(value, cfg));
            }
                return;
            case "BrickletJoystickPositionY": {
                BrickletJoystick.Position value = this.getPosition();
                if (true)
                    updateStateFn.accept(channel, transformBrickletJoystickPositionYGetter0(value, cfg));
            }
                return;
            case "BrickletJoystickPressed": {
                boolean value = this.isPressed();
                if (true)
                    triggerChannelFn.accept(channel, transformBrickletJoystickPressedGetter0(value, cfg));
            }
                return;
            case "BrickletJoystickAnalogValueX": {
                BrickletJoystick.AnalogValue value = this.getAnalogValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletJoystickAnalogValueXGetter0(value, cfg));
            }
                return;
            case "BrickletJoystickAnalogValueY": {
                BrickletJoystick.AnalogValue value = this.getAnalogValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletJoystickAnalogValueYGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletJoystickPositionXCallback0(short x, short y,
            BrickletJoystickConfig cfg) {
        return new DecimalType(x);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletJoystickPositionYCallback0(short x, short y,
            BrickletJoystickConfig cfg) {
        return new DecimalType(y);
    }

    private String transformBrickletJoystickPressedCallback0(BrickletJoystickConfig cfg) {
        return CommonTriggerEvents.PRESSED;
    }

    private String transformBrickletJoystickPressedCallback1(BrickletJoystickConfig cfg) {
        return CommonTriggerEvents.RELEASED;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletJoystickAnalogValueXCallback0(int x, int y,
            BrickletJoystickConfig cfg) {
        return new DecimalType(x);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletJoystickAnalogValueYCallback0(int x, int y,
            BrickletJoystickConfig cfg) {
        return new DecimalType(x);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletJoystickPositionXGetter0(
            BrickletJoystick.Position value, BrickletJoystickConfig cfg) {
        return new DecimalType(value.x);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletJoystickPositionYGetter0(
            BrickletJoystick.Position value, BrickletJoystickConfig cfg) {
        return new DecimalType(value.y);
    }

    private String transformBrickletJoystickPressedGetter0(boolean value, BrickletJoystickConfig cfg) {
        return value ? CommonTriggerEvents.PRESSED : CommonTriggerEvents.RELEASED;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletJoystickAnalogValueXGetter0(
            BrickletJoystick.AnalogValue value, BrickletJoystickConfig cfg) {
        return new DecimalType(value.x);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletJoystickAnalogValueYGetter0(
            BrickletJoystick.AnalogValue value, BrickletJoystickConfig cfg) {
        return new DecimalType(value.y);
    }

}
