/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.StringType;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletOLED128x64V2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the OLED 128x64 Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletOLED128x64V2Wrapper extends BrickletOLED128x64V2 implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletOLED128x64V2.DEVICE_DISPLAY_NAME,
            "brickletoled128x64v2", BrickletOLED128x64V2.DEVICE_IDENTIFIER, BrickletOLED128x64V2Wrapper.class,
            BrickletOLED128x64V2Actions.class, "2.0.0", true);

    public BrickletOLED128x64V2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletOLED128x64V2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletOLED128x64V2Config cfg = (BrickletOLED128x64V2Config) config.as(BrickletOLED128x64V2Config.class);

        this.setDisplayConfiguration(cfg.contrast.shortValue(), cfg.invert, cfg.automaticDraw);
        this.setStatusLEDConfig(cfg.statusLEDConfig);
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletOLED128x64V2Text");
        result.add("BrickletOLED128x64V2ClearDisplay");
        result.add("BrickletOLED128x64V2DrawBufferedFrame");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletOLED128x64V2Text":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletOLED128x64V2Text"), "Text", "String")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletOLED128x64V2Text"))
                        .withDescription(
                                "Text to display on the LCD. Command format is [line],[position],[text].<br/><br/>Additional ',' are handled as part of the text. Unicode characters are converted to the LCD character set if possible. Additionally you can use \\x[two hex digits] to use a character of the LCD character set directly.")
                        .build();
            case "BrickletOLED128x64V2ClearDisplay":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletOLED128x64V2ClearDisplay"), "Clear Display",
                                "String")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletOLED128x64V2ClearDisplay"))
                        .withDescription("Deletes all characters from the display.")
                        .withCommandDescription(CommandDescriptionBuilder.create()
                                .withCommandOption(new CommandOption("CLEAR", "Clear")).build())
                        .build();
            case "BrickletOLED128x64V2DrawBufferedFrame":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletOLED128x64V2DrawBufferedFrame"),
                                "Draw Buffered Frame", "String")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletOLED128x64V2DrawBufferedFrame"))
                        .withDescription("Draws the currently buffered frame.")
                        .withCommandDescription(CommandDescriptionBuilder.create()
                                .withCommandOption(new CommandOption("DRAW", "Draw")).build())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge OLED 128x64 Bricklet 2.0").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("3.3cm (1.3\") OLED display with 128x64 pixels")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletOLED128x64V2Text",
                                new ChannelTypeUID("tinkerforge", "BrickletOLED128x64V2Text")).withLabel("Text")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletOLED128x64V2ClearDisplay",
                                new ChannelTypeUID("tinkerforge", "BrickletOLED128x64V2ClearDisplay"))
                                        .withLabel("Clear Display").build(),
                        new ChannelDefinitionBuilder("BrickletOLED128x64V2DrawBufferedFrame",
                                new ChannelTypeUID("tinkerforge", "BrickletOLED128x64V2DrawBufferedFrame"))
                                        .withLabel("Draw Buffered Frame").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletoled128x64v2":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(
                                ConfigDescriptionParameterBuilder.create("contrast", Type.INTEGER).withDefault("143")
                                        .withDescription(
                                                "Sets the contrast of the display (0-255).")
                                        .withLabel("Contrast").withAdvanced(true).withMinimum(BigDecimal
                                                .valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(255)).build(),
                                ConfigDescriptionParameterBuilder.create("invert", Type.BOOLEAN).withDefault("false")
                                        .withDescription("Inverts the color (black/white) of the display.")
                                        .withLabel("Invert").withAdvanced(true).build(),
                                ConfigDescriptionParameterBuilder.create("automaticDraw", Type.BOOLEAN)
                                        .withDefault("true")
                                        .withDescription(
                                                "If automatic draw is enabled, the display is automatically updated when writing text or clearing the display. If it is disabled, the changes are written into an internal buffer and only shown on the display after triggering the Draw Buffered Frame channel.")
                                        .withLabel("Automatic Draw").withAdvanced(true).build(),
                                ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER)
                                        .withDefault("3")
                                        .withDescription(
                                                "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                        .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                        .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                                new ParameterOption("1", "On"),
                                                new ParameterOption("2", "Show Heartbeat"),
                                                new ParameterOption("3", "Show Status")))
                                        .build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletOLED128x64V2Text":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletOLED128x64V2ClearDisplay":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletOLED128x64V2DrawBufferedFrame":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        switch (channel) {
            case "BrickletOLED128x64V2Text":
                return;
            case "BrickletOLED128x64V2ClearDisplay":
                return;
            case "BrickletOLED128x64V2DrawBufferedFrame":
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            case "BrickletOLED128x64V2Text":
                if (command instanceof StringType) {
                    StringType cmd = (StringType) command;
                    this.writeLine(Helper.parseDisplayCommandLine(cmd.toString(), logger),
                            Helper.parseDisplayCommandPosition(cmd.toString(), logger),
                            Helper.parseDisplayCommandText(cmd.toString(), logger, false));
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletOLED128x64V2ClearDisplay":
                if (command instanceof StringType) {
                    this.clearDisplay();
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletOLED128x64V2DrawBufferedFrame":
                if (command instanceof StringType) {
                    this.drawBufferedFrame(true);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }

                break;
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

}
