/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletPTC;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the PTC Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletPTCActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletPTCGetTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPTCWrapper dev = ((@Nullable BrickletPTCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletPTCGetTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPTCActions) {
            return ((BrickletPTCActions) actions).brickletPTCGetTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCActions class.");
        }
    }

    @RuleAction(label = "Is Sensor Connected")
    public @ActionOutput(name = "connected", type = "boolean") Map<String, Object> brickletPTCIsSensorConnected()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPTCWrapper dev = ((@Nullable BrickletPTCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean value = dev.isSensorConnected();

        result.put("connected", value);
        return result;
    }

    public static Map<String, Object> brickletPTCIsSensorConnected(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPTCActions) {
            return ((BrickletPTCActions) actions).brickletPTCIsSensorConnected();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCActions class.");
        }
    }

    @RuleAction(label = "Get Wire Mode")
    public @ActionOutput(name = "mode", type = "short") Map<String, Object> brickletPTCGetWireMode()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPTCWrapper dev = ((@Nullable BrickletPTCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        short value = dev.getWireMode();

        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletPTCGetWireMode(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPTCActions) {
            return ((BrickletPTCActions) actions).brickletPTCGetWireMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCActions class.");
        }
    }

    @RuleAction(label = "Get Resistance")
    public @ActionOutput(name = "resistance", type = "int") Map<String, Object> brickletPTCGetResistance()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPTCWrapper dev = ((@Nullable BrickletPTCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getResistance();

        result.put("resistance", value);
        return result;
    }

    public static Map<String, Object> brickletPTCGetResistance(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPTCActions) {
            return ((BrickletPTCActions) actions).brickletPTCGetResistance();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCActions class.");
        }
    }

    @RuleAction(label = "Get Noise Rejection Filter")
    public @ActionOutput(name = "filter", type = "short") Map<String, Object> brickletPTCGetNoiseRejectionFilter()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPTCWrapper dev = ((@Nullable BrickletPTCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        short value = dev.getNoiseRejectionFilter();

        result.put("filter", value);
        return result;
    }

    public static Map<String, Object> brickletPTCGetNoiseRejectionFilter(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPTCActions) {
            return ((BrickletPTCActions) actions).brickletPTCGetNoiseRejectionFilter();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletPTCGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletPTCWrapper dev = ((@Nullable BrickletPTCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletPTC.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletPTCGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletPTCActions) {
            return ((BrickletPTCActions) actions).brickletPTCGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCActions class.");
        }
    }
}
