/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletRotaryEncoder;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Rotary Encoder Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRotaryEncoderActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Count")
    public @ActionOutput(name = "count", type = "int") Map<String, Object> brickletRotaryEncoderGetCount(
            @ActionInput(name = "reset") boolean reset) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRotaryEncoderWrapper dev = ((@Nullable BrickletRotaryEncoderWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getCount(reset);

        result.put("count", value);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderGetCount(@Nullable ThingActions actions, boolean reset)
            throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderActions) {
            return ((BrickletRotaryEncoderActions) actions).brickletRotaryEncoderGetCount(reset);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderActions class.");
        }
    }

    @RuleAction(label = "Is Pressed")
    public @ActionOutput(name = "pressed", type = "boolean") Map<String, Object> brickletRotaryEncoderIsPressed()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRotaryEncoderWrapper dev = ((@Nullable BrickletRotaryEncoderWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean value = dev.isPressed();

        result.put("pressed", value);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderIsPressed(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderActions) {
            return ((BrickletRotaryEncoderActions) actions).brickletRotaryEncoderIsPressed();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletRotaryEncoderGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRotaryEncoderWrapper dev = ((@Nullable BrickletRotaryEncoderWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletRotaryEncoder.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRotaryEncoderGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRotaryEncoderActions) {
            return ((BrickletRotaryEncoderActions) actions).brickletRotaryEncoderGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRotaryEncoderActions class.");
        }
    }
}
