/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletSegmentDisplay4x7V2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Segment Display 4x7 Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletSegmentDisplay4x7V2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Set Segments")
    public void brickletSegmentDisplay4x7V2SetSegments(@ActionInput(name = "digit0") boolean[] digit0,
            @ActionInput(name = "digit1") boolean[] digit1, @ActionInput(name = "digit2") boolean[] digit2,
            @ActionInput(name = "digit3") boolean[] digit3, @ActionInput(name = "colon") boolean[] colon,
            @ActionInput(name = "tick") boolean tick) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletSegmentDisplay4x7V2Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7V2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setSegments(digit0, digit1, digit2, digit3, colon, tick);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2Segments")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2Segments"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2ColonUpper")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2ColonUpper"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2ColonLower")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2ColonLower"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2Tick")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2Tick"), RefreshType.REFRESH);
        }
    }

    public static void brickletSegmentDisplay4x7V2SetSegments(@Nullable ThingActions actions, boolean[] digit0,
            boolean[] digit1, boolean[] digit2, boolean[] digit3, boolean[] colon, boolean tick)
            throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2SetSegments(digit0, digit1,
                    digit2, digit3, colon, tick);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get Segments")
    public @ActionOutput(name = "digit0", type = "boolean[]") @ActionOutput(name = "digit1", type = "boolean[]") @ActionOutput(name = "digit2", type = "boolean[]") @ActionOutput(name = "digit3", type = "boolean[]") @ActionOutput(name = "colon", type = "boolean[]") @ActionOutput(name = "tick", type = "boolean") Map<String, Object> brickletSegmentDisplay4x7V2GetSegments()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSegmentDisplay4x7V2Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletSegmentDisplay4x7V2.Segments value = dev.getSegments();

        result.put("digit0", value.digit0);
        result.put("digit1", value.digit1);
        result.put("digit2", value.digit2);
        result.put("digit3", value.digit3);
        result.put("colon", value.colon);
        result.put("tick", value.tick);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2GetSegments(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2GetSegments();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Set Selected Segment")
    public void brickletSegmentDisplay4x7V2SetSelectedSegment(@ActionInput(name = "segment") int segment,
            @ActionInput(name = "value") boolean value) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletSegmentDisplay4x7V2Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7V2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setSelectedSegment(segment, value);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2Segments")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2Segments"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2ColonUpper")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2ColonUpper"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2ColonLower")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2ColonLower"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2Tick")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2Tick"), RefreshType.REFRESH);
        }
    }

    public static void brickletSegmentDisplay4x7V2SetSelectedSegment(@Nullable ThingActions actions, int segment,
            boolean value) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2SetSelectedSegment(segment,
                    value);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get Selected Segment")
    public @ActionOutput(name = "value", type = "boolean") Map<String, Object> brickletSegmentDisplay4x7V2GetSelectedSegment(
            @ActionInput(name = "segment") int segment) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSegmentDisplay4x7V2Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean value = dev.getSelectedSegment(segment);

        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2GetSelectedSegment(@Nullable ThingActions actions,
            int segment) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions)
                    .brickletSegmentDisplay4x7V2GetSelectedSegment(segment);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Set Brightness")
    public void brickletSegmentDisplay4x7V2SetBrightness(@ActionInput(name = "brightness") int brightness)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletSegmentDisplay4x7V2Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7V2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setBrightness(brightness);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2Brightness")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2Brightness"), RefreshType.REFRESH);
        }
    }

    public static void brickletSegmentDisplay4x7V2SetBrightness(@Nullable ThingActions actions, int brightness)
            throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2SetBrightness(brightness);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get Brightness")
    public @ActionOutput(name = "brightness", type = "int") Map<String, Object> brickletSegmentDisplay4x7V2GetBrightness()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSegmentDisplay4x7V2Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getBrightness();

        result.put("brightness", value);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2GetBrightness(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2GetBrightness();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Start Counter")
    public void brickletSegmentDisplay4x7V2StartCounter(@ActionInput(name = "valueFrom") int valueFrom,
            @ActionInput(name = "valueTo") int valueTo, @ActionInput(name = "increment") int increment,
            @ActionInput(name = "length") long length) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickletSegmentDisplay4x7V2Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7V2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.startCounter(valueFrom, valueTo, increment, length);

    }

    public static void brickletSegmentDisplay4x7V2StartCounter(@Nullable ThingActions actions, int valueFrom,
            int valueTo, int increment, long length) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2StartCounter(valueFrom, valueTo,
                    increment, length);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get Counter Value")
    public @ActionOutput(name = "value", type = "int") Map<String, Object> brickletSegmentDisplay4x7V2GetCounterValue()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSegmentDisplay4x7V2Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getCounterValue();

        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2GetCounterValue(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2GetCounterValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Set Numeric Value")
    public void brickletSegmentDisplay4x7V2SetNumericValue(@ActionInput(name = "value") int[] value)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletSegmentDisplay4x7V2Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7V2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setNumericValue(value);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2Segments")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2Segments"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2ColonUpper")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2ColonUpper"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2ColonLower")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2ColonLower"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2Tick")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSegmentDisplay4x7V2Tick"), RefreshType.REFRESH);
        }
    }

    public static void brickletSegmentDisplay4x7V2SetNumericValue(@Nullable ThingActions actions, int[] value)
            throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2SetNumericValue(value);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletSegmentDisplay4x7V2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSegmentDisplay4x7V2Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletSegmentDisplay4x7V2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSegmentDisplay4x7V2Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletSegmentDisplay4x7V2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSegmentDisplay4x7V2Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletSegmentDisplay4x7V2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSegmentDisplay4x7V2Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletSegmentDisplay4x7V2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "int[]") @ActionOutput(name = "firmwareVersion", type = "int[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletSegmentDisplay4x7V2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSegmentDisplay4x7V2Wrapper dev = ((@Nullable BrickletSegmentDisplay4x7V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletSegmentDisplay4x7V2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }
}
