/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletTemperatureIRV2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Temperature IR Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletTemperatureIRV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Ambient Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletTemperatureIRV2GetAmbientTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletTemperatureIRV2Wrapper dev = ((@Nullable BrickletTemperatureIRV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getAmbientTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRV2GetAmbientTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRV2Actions) {
            return ((BrickletTemperatureIRV2Actions) actions).brickletTemperatureIRV2GetAmbientTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Object Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletTemperatureIRV2GetObjectTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletTemperatureIRV2Wrapper dev = ((@Nullable BrickletTemperatureIRV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getObjectTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRV2GetObjectTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRV2Actions) {
            return ((BrickletTemperatureIRV2Actions) actions).brickletTemperatureIRV2GetObjectTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Emissivity")
    public @ActionOutput(name = "emissivity", type = "int") Map<String, Object> brickletTemperatureIRV2GetEmissivity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletTemperatureIRV2Wrapper dev = ((@Nullable BrickletTemperatureIRV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getEmissivity();

        result.put("emissivity", value);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRV2GetEmissivity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRV2Actions) {
            return ((BrickletTemperatureIRV2Actions) actions).brickletTemperatureIRV2GetEmissivity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletTemperatureIRV2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletTemperatureIRV2Wrapper dev = ((@Nullable BrickletTemperatureIRV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRV2ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRV2Actions) {
            return ((BrickletTemperatureIRV2Actions) actions).brickletTemperatureIRV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletTemperatureIRV2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletTemperatureIRV2Wrapper dev = ((@Nullable BrickletTemperatureIRV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRV2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRV2Actions) {
            return ((BrickletTemperatureIRV2Actions) actions).brickletTemperatureIRV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletTemperatureIRV2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletTemperatureIRV2Wrapper dev = ((@Nullable BrickletTemperatureIRV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRV2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRV2Actions) {
            return ((BrickletTemperatureIRV2Actions) actions).brickletTemperatureIRV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletTemperatureIRV2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletTemperatureIRV2Wrapper dev = ((@Nullable BrickletTemperatureIRV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletTemperatureIRV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRV2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRV2Actions) {
            return ((BrickletTemperatureIRV2Actions) actions).brickletTemperatureIRV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "int[]") @ActionOutput(name = "firmwareVersion", type = "int[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletTemperatureIRV2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletTemperatureIRV2Wrapper dev = ((@Nullable BrickletTemperatureIRV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletTemperatureIRV2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRV2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRV2Actions) {
            return ((BrickletTemperatureIRV2Actions) actions).brickletTemperatureIRV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRV2Actions class.");
        }
    }
}
