/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletTemperatureIR;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Temperature IR Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletTemperatureIRWrapper extends BrickletTemperatureIR implements StandardFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletTemperatureIR.DEVICE_DISPLAY_NAME,
            "bricklettemperatureir", BrickletTemperatureIR.DEVICE_IDENTIFIER, BrickletTemperatureIRWrapper.class,
            BrickletTemperatureIRActions.class, "2.0.0", false);

    public BrickletTemperatureIRWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletTemperatureIRWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletTemperatureIRConfig cfg = (BrickletTemperatureIRConfig) config.as(BrickletTemperatureIRConfig.class);
        this.addAmbientTemperatureListener(this.reg((temperature) -> {
            if (true) {
                updateStateFn.accept("BrickletTemperatureIRAmbientTemperature",
                        transformBrickletTemperatureIRAmbientTemperatureCallback0(temperature, cfg));
            }
        }, this::removeAmbientTemperatureListener));
        this.addObjectTemperatureListener(this.reg((temperature) -> {
            if (true) {
                updateStateFn.accept("BrickletTemperatureIRObjectTemperature",
                        transformBrickletTemperatureIRObjectTemperatureCallback0(temperature, cfg));
            }
        }, this::removeObjectTemperatureListener));

        {
            BrickletTemperatureIRAmbientTemperatureConfig channelCfg = getChannelConfigFn
                    .apply("BrickletTemperatureIRAmbientTemperature")
                    .as(BrickletTemperatureIRAmbientTemperatureConfig.class);
            this.setAmbientTemperatureCallbackPeriod(channelCfg.updateInterval);
            this.setAmbientTemperatureCallbackThreshold('x', (short) 0, (short) 0);
        }
        {
            BrickletTemperatureIRObjectTemperatureConfig channelCfg = getChannelConfigFn
                    .apply("BrickletTemperatureIRObjectTemperature")
                    .as(BrickletTemperatureIRObjectTemperatureConfig.class);
            this.setObjectTemperatureCallbackPeriod(channelCfg.updateInterval);
            this.setObjectTemperatureCallbackThreshold('x', (short) 0, (short) 0);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setAmbientTemperatureCallbackPeriod(0);
        this.setObjectTemperatureCallbackPeriod(0);

    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletTemperatureIRAmbientTemperature");
        result.add("BrickletTemperatureIRObjectTemperature");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletTemperatureIRAmbientTemperature":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletTemperatureIRAmbientTemperature"),
                                "Ambient Temperature", "Number:Temperature")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletTemperatureIRAmbientTemperature"))
                        .withDescription("Measured ambient temperature")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(-40))
                                        .withMaximum(BigDecimal.valueOf(125)).withStep(BigDecimal.valueOf(0.1))
                                        .withPattern("%.1f %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletTemperatureIRObjectTemperature":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletTemperatureIRObjectTemperature"),
                                "Object Temperature", "Number:Temperature")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletTemperatureIRObjectTemperature"))
                        .withDescription(
                                "Measured object temperature, i.e. the temperature of the surface of the object the sensor is aimed at. The temperature of different materials is dependent on their emissivity: https://en.wikipedia.org/wiki/Emissivity.")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(-70))
                                        .withMaximum(BigDecimal.valueOf(380)).withStep(BigDecimal.valueOf(0.1))
                                        .withPattern("%.1f %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Temperature IR Bricklet").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription(
                        "Measures contactless object temperature between -70°C and +380°C")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletTemperatureIRAmbientTemperature",
                                new ChannelTypeUID("tinkerforge", "BrickletTemperatureIRAmbientTemperature"))
                                        .withLabel("Ambient Temperature").build(),
                        new ChannelDefinitionBuilder("BrickletTemperatureIRObjectTemperature",
                                new ChannelTypeUID("tinkerforge", "BrickletTemperatureIRObjectTemperature"))
                                        .withLabel("Object Temperature").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:bricklettemperatureir":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletTemperatureIRAmbientTemperature":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletTemperatureIRObjectTemperature":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletTemperatureIRConfig cfg = (BrickletTemperatureIRConfig) config.as(BrickletTemperatureIRConfig.class);
        switch (channel) {
            case "BrickletTemperatureIRAmbientTemperature": {
                short value = this.getAmbientTemperature();
                if (true)
                    updateStateFn.accept(channel, transformBrickletTemperatureIRAmbientTemperatureGetter0(value, cfg));
            }
                return;
            case "BrickletTemperatureIRObjectTemperature": {
                short value = this.getObjectTemperature();
                if (true)
                    updateStateFn.accept(channel, transformBrickletTemperatureIRObjectTemperatureGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletTemperatureIRAmbientTemperatureCallback0(
            short temperature, BrickletTemperatureIRConfig cfg) {
        return new QuantityType<>(temperature / 10.0, SIUnits.CELSIUS);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletTemperatureIRObjectTemperatureCallback0(
            short temperature, BrickletTemperatureIRConfig cfg) {
        return new QuantityType<>(temperature / 10.0, SIUnits.CELSIUS);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletTemperatureIRAmbientTemperatureGetter0(short value,
            BrickletTemperatureIRConfig cfg) {
        return new QuantityType<>(value / 10.0, SIUnits.CELSIUS);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletTemperatureIRObjectTemperatureGetter0(short value,
            BrickletTemperatureIRConfig cfg) {
        return new QuantityType<>(value / 10.0, SIUnits.CELSIUS);
    }

}
