/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-08.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletThermocouple;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Thermocouple Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletThermocoupleActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletThermocoupleGetTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermocoupleWrapper dev = ((@Nullable BrickletThermocoupleWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleGetTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleActions) {
            return ((BrickletThermocoupleActions) actions).brickletThermocoupleGetTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleActions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "averaging", type = "short") @ActionOutput(name = "thermocoupleType", type = "short") @ActionOutput(name = "filter", type = "short") Map<String, Object> brickletThermocoupleGetConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermocoupleWrapper dev = ((@Nullable BrickletThermocoupleWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletThermocouple.Configuration value = dev.getConfiguration();

        result.put("averaging", value.averaging);
        result.put("thermocoupleType", value.thermocoupleType);
        result.put("filter", value.filter);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleGetConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleActions) {
            return ((BrickletThermocoupleActions) actions).brickletThermocoupleGetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleActions class.");
        }
    }

    @RuleAction(label = "Get Error State")
    public @ActionOutput(name = "overUnder", type = "boolean") @ActionOutput(name = "openCircuit", type = "boolean") Map<String, Object> brickletThermocoupleGetErrorState()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermocoupleWrapper dev = ((@Nullable BrickletThermocoupleWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletThermocouple.ErrorState value = dev.getErrorState();

        result.put("overUnder", value.overUnder);
        result.put("openCircuit", value.openCircuit);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleGetErrorState(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleActions) {
            return ((BrickletThermocoupleActions) actions).brickletThermocoupleGetErrorState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletThermocoupleGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletThermocoupleWrapper dev = ((@Nullable BrickletThermocoupleWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletThermocouple.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleActions) {
            return ((BrickletThermocoupleActions) actions).brickletThermocoupleGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleActions class.");
        }
    }
}
